import pygame
import math

from water import Water

import util

from locals import *

class Enemy(pygame.sprite.Sprite):
    def __init__(self, pos = [0.0, 0.0], health = 1):
        pygame.sprite.Sprite.__init__(self)

        self.pos = pos
        self.velocity [0.0, 0.0]

        self.health = health

        #self.hitmask = pygame.surfarray.array_alpha(self.image)

        self.angle = 0
        self.targetangle = 0
        self.dying = False
        self.dead = False

        self.t = 0

    def damage(self):
        self.health -= 1

        if self.health == 0:
            self.die()

    def update(self):
        self.t += 1

        self.velocity[1] += 1.0 # Gravity

        self.pos[0] += self.velocity[0]
        self.pos[1] += self.velocity[1]

    def die(self):
        self.dying = True



 #!/usr/bin/python

import pygame
import math
import random
import sys

import PixelPerfect

from pygame.locals import *

from water import Water
from menu import Menu
from game import Game
from highscores import Highscores
from options import Options

import util

from locals import *

import health
import cloud
import mine
import steamboat
import pirateboat
import shark
import seagull

def init():
    health.init()
    steamboat.init()
    shark.init()
    pirateboat.init()
    cloud.init()
    mine.init()
    seagull.init()

def main():
    global SCREEN_FULLSCREEN
    pygame.init()

    util.load_config()

    if len(sys.argv) > 1:
        for arg in sys.argv:
            if arg == "-np":
                Variables.particles = False
            elif arg == "-na":
                Variables.alpha = False
            elif arg == "-nm":
                Variables.music = False
            elif arg == "-ns":
                Variables.sound = False
            elif arg == "-f":
                SCREEN_FULLSCREEN = True

    scr_options = 0
    if SCREEN_FULLSCREEN: scr_options += FULLSCREEN
    screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT),scr_options ,32)

    pygame.display.set_icon(util.load_image("kuvake"))
    pygame.display.set_caption("Trip on the Funny Boat")

    init()

    joy = None
    if pygame.joystick.get_count() > 0:
        joy = pygame.joystick.Joystick(0)
        joy.init()

    try:
        util.load_music("JDruid-Trip_on_the_Funny_Boat")
        if Variables.music:
            pygame.mixer.music.play(-1)
    except:
        # It's not a critical problem if there's no music
        pass

    pygame.time.set_timer(NEXTFRAME, 1000 / FPS) # 30 fps

    Water.global_water = Water()

    main_selection = 0

    while True:
        main_selection = Menu(screen, ("New Game", "High Scores", "Options", "Quit"), main_selection).run()
        if main_selection == 0:
            # New Game
            selection = Menu(screen, ("Story Mode", "Endless Mode")).run()
            if selection == 0:
                # Story
                score = Game(screen).run()
                Highscores(screen, score).run()
            elif selection == 1:
                # Endless
                score = Game(screen, True).run()
                Highscores(screen, score, True).run()
        elif main_selection == 1:
            # High Scores
            selection = 0
            while True:
                selection = Menu(screen, ("Story Mode", "Endless Mode", "Endless Online"), selection).run()
                if selection == 0:
                    # Story
                    Highscores(screen).run()
                elif selection == 1:
                    # Endless
                    Highscores(screen, endless = True).run()
                elif selection == 2:
                    # Online
                    Highscores(screen, endless = True, online = True).run()
                else:
                    break
        elif main_selection == 2:
            # Options
            selection = Options(screen).run()
        else: #if main_selection == 3:
            # Quit
            return


if __name__ == '__main__':
    main()



import pygame
import random
from pygame.locals import *

from locals import *

class Particle (pygame.sprite.Sprite):
    def __init__(self, position, vect, colour, acceleration, size, life, opacity, underwater = True):
        pygame.sprite.Sprite.__init__(self)

        self.rect = pygame.Rect(position[0], position[1], size, size)
        self.vect = vect
        self.colour = colour
        self.acceleration = acceleration
        self.initial_life = life
        self.life = life
        self.opacity = opacity
        self.underwater = underwater

        self.image = pygame.Surface([int(size), int(size)])#, SRCALPHA, 32)
        self.image.fill((255,0,255))
        self.image.set_colorkey((255,0,255))

        pygame.draw.ellipse(self.image, self.colour, self.image.get_rect())

        if Variables.alpha:
            self.image.set_alpha(self.life * 255 * self.opacity / self.initial_life)

    def update(self):
        self.rect.left += self.vect[0]
        self.rect.top += self.vect[1]
        self.vect[0] += self.acceleration[0]
        self.vect[1] += self.acceleration[1]
        if self.life > 0:
            self.life -= 1

        if not self.underwater and self.vect[1] > 0.0:
            self.life = 0

        if Variables.alpha:
            self.image.set_alpha(self.life * 255 * self.opacity / self.initial_life)

class Particles (pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)

        self.particles = []
        self.particle_sprites = pygame.sprite.Group()
        if Variables.alpha:
            self.image = pygame.Surface((SCREEN_WIDTH, SCREEN_HEIGHT), SRCALPHA, 32)
        else:
            self.image = pygame.Surface((SCREEN_WIDTH, SCREEN_HEIGHT))
            self.image.set_colorkey((255,0,255))
        self.rect = self.image.get_rect()

    def update(self):
        self.image.fill((255,0,255,0))
        
        self.particle_sprites.draw(self.image)

        for p in self.particles:
            p.update()
            if p.life <= 0:
                self.particles.remove(p)
                self.particle_sprites.remove(p)

    def add_blood_particle(self, position):
        particle = Particle(position, [random.random() * 5 - 2.5, random.random() * 5 - 2.5], [230, 30, 20], [0.0, 0.7], random.random() * 5.0 + 1.0, random.random() * 30, 1.0)
        self.particles.append(particle)
        self.particle_sprites.add(particle)

    def add_explosion_particle(self, position):
        particle = Particle(position, [random.random() * 5 - 2.5, random.random() * 5 - 2.5], [230, 30 + random.random() * 200, 20], [0.0, 0.2], random.random() * 7.0 + 1.0, random.random() * 30, 1.0)
        self.particles.append(particle)
        self.particle_sprites.add(particle)

    def add_water_particle(self, position):
        particle = Particle(position, [random.random() * 5 - 2.5, -random.random() * 2.5 - 2.0], (20,60,180), (0.0, 0.3), random.random() * 5.0 + 1.0, random.random() * 30, 0.5, underwater = False)
        self.particles.append(particle)
        self.particle_sprites.add(particle)

    def add_debris_particle(self, position):
        particle = Particle(position, [random.random() * 5 - 2.5, random.random() * 5 - 2.5], [90, 90, 90], [0.0, 0.2], random.random() * 7.0 + 1.0, random.random() * 30, 1.0)
        self.particles.append(particle)
        self.particle_sprites.add(particle)

    def add_wood_particle(self, position):
        particle = Particle(position, [random.random() * 5 - 2.5, random.random() * 5 - 2.5], [148, 69, 6], [0.0, 0.2], random.random() * 7.0 + 1.0, random.random() * 30, 1.0)
        self.particles.append(particle)
        self.particle_sprites.add(particle)

    def add_steam_particle(self, position):
        particle = Particle(position, [-random.random() * 0.3, -random.random() * 0.1], [240, 240, 240], [-0.1, -0.00002], random.random() * 10.0 + 1.0, random.random() * 30, 0.5)
        self.particles.append(particle)
        self.particle_sprites.add(particle)

    def add_fire_steam_particle(self, position):
        particle = Particle(position, [-random.random() * 0.3, -random.random() * 0.1], [255, 210, 170], [-0.1, -0.00002], random.random() * 11.0 + 1.0, random.random() * 30, 0.4, False)
        self.particles.append(particle)
        self.particle_sprites.add(particle)

    def add_trace_particle(self, position):
        particle = Particle(position, [0.0, 0.0], [170, 170, 170], [0.0, 0.0], 6.0, 5+random.random() * 5, 0.1+random.random()*0.1, False)
        self.particles.append(particle)
        self.particle_sprites.add(particle)



import pygame
import codecs
import os
from locals import *

# Some general utility functions here

pygame.font.init()

# The Cosmetica font seemed to be broken... had trouble rendering small text
# with underscores (_) ... it's weird.
smallfont = pygame.font.Font("data/Vera.ttf", 14)
bigfont = pygame.font.Font("data/Vera.ttf", 24)

def get_config_path():
    pathname = ""
    try:
        pathname = os.environ["HOME"] + "/.funnyboat"
    except:
        try:
            pathname = os.environ["APPDATA"] + "/Funny Boat"
        except:
            print "Couldn't get environment variable for home directory"
            pathname = "."
    if not os.path.exists(pathname):
        os.mkdir(pathname)
    return pathname

def str_to_bool(text):
    ltext = text.lower().strip()
    if ltext in ("on", "true", "yes"):
        return True
    else:
        return False

def load_config():
    filename = get_config_path() + "/config"
    if not os.path.exists(filename):
        return
    f = codecs.open(filename, "r", "utf_8")

    for line in f.readlines():
        parts = line.split("\t")
        if len(parts) < 2:
            continue
        variable,value = parts
        variable = variable.lower()
        if variable == "alpha":
            pass        
            # This is disabled because it's no longer in the in-game options
            # and it seems to be acting up in weird ways, so it's better
            # to just ditch it. The code is littered with it still, so I'm
            # not completely removing the variable yet.
            #Variables.alpha = str_to_bool(value)
        elif variable == "particles":
            Variables.particles = str_to_bool(value)
        elif variable == "music":
            Variables.music = str_to_bool(value)
        elif variable == "name":
            Variables.name = value.strip()[:32]
        elif variable == "sound":
            Variables.sound = str_to_bool(value)
        elif variable == "aa":
            Variables.aa = str_to_bool(value)

    f.close()

def save_config():
    filename = get_config_path() + "/config"
    if not os.path.exists(get_config_path()):
        os.mkdir(get_config_path())
    f = codecs.open(filename, "w", "utf_8")

    print >> f, "alpha\t%s" % Variables.alpha
    print >> f, "particles\t%s" % Variables.particles
    print >> f, "music\t%s" % Variables.music
    print >> f, "name\t%s" % Variables.name
    print >> f, "sound\t%s" % Variables.sound
    print >> f, "aa\t%s" % Variables.aa

    f.close()

def load_image(name):
    image = pygame.image.load("data/" + name + ".png").convert_alpha()
    if not Variables.alpha:
        # this is kinda not useful. you can't even get Variables.alpha set
        # without modifying the config file by hand
        # in case someone does, at least it won't crash the game if they don't
        # have Numeric installed
        w,h = image.get_size()
        for i in xrange(w):
            for j in xrange(h):
                pixel = image.get_at((i,j))
                if pixel[3] > 127:
                    image.set_at((i,j), (pixel[0], pixel[1], pixel[2], 255))
                else:
                    image.set_at((i,j), (pixel[0], pixel[1], pixel[2], 0))
        #surfarray = pygame.surfarray.pixels_alpha(image)
        #for i in xrange(len(surfarray)):
        #    for j in xrange(len(surfarray[i])):
        #        if surfarray[i][j] > 127:
        #            surfarray[i][j] = 255
        #        else:
        #            surfarray[i][j] = 0

    return image

def load_sound(name):
    return pygame.mixer.Sound("data/" + name + ".ogg")

def load_music(name):
    # The all-caps ogg is because the original file just happened to be that way
    pygame.mixer.music.load("data/" + name + ".ogg")

def rotate(surf, angle):
    if Variables.aa:
        return pygame.transform.rotozoom(surf, angle, 1.0)
    else:
        return pygame.transform.rotate(surf, angle)




import pygame
from pygame.locals import *

from locals import *

import util

def init():
    Health.heart = util.load_image("sydan")
    Health.heart_empty = util.load_image("sydan-tyhja")
    Health.heart_broken = util.load_image("sydan-rikki")
    heart_broken = pygame.Surface((Health.heart_broken.get_rect().width, Health.heart_broken.get_rect().height))
    heart_broken.fill((255,0,255))
    heart_broken.set_colorkey((255,0,255))
    heart_broken.blit(Health.heart_broken, heart_broken.get_rect())
    Health.heart_broken = heart_broken

class Health (pygame.sprite.Sprite):
    heart = None
    heart_empty = None
    heart_broken = None
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)

        if not Health.heart or not Health.heart_empty or not Health.heart_broken:
            init()

        self.rect = pygame.Rect(10, 0, Health.heart.get_width() * 5 + 4, Health.heart.get_height() * 2)
        self.image = pygame.Surface((self.rect.width, self.rect.height))
        self.image.set_colorkey((255,255,255))

        self.hearts_left = 5
        self.hearts_dying = 0
        self.hearts_deathcounters = [0,0,0,0,0]


        self.update()

    def update(self):
        # clear the surface
        self.image.fill((255,255,255))

        for i in xrange(5):
            rect = pygame.Rect(i * (Health.heart.get_width() + 1), Health.heart.get_height(), Health.heart.get_width(), Health.heart.get_height())
            if i < self.hearts_left:
                self.image.blit(Health.heart, rect)
            else:
                self.image.blit(Health.heart_empty, rect)
                if i < (self.hearts_left + self.hearts_dying):
                    rect.top -= self.hearts_deathcounters[i]
                    if Variables.alpha:
                        Health.heart_broken.set_alpha(255 - self.hearts_deathcounters[i]*10)
                    self.image.blit(Health.heart_broken, rect)
                    self.hearts_deathcounters[i] += 1
                    if self.hearts_deathcounters[i] == 25:
                        self.hearts_dying -= 1


    def damage(self):
        if self.hearts_left > 0:
            self.hearts_left -= 1
            self.hearts_dying += 1
        self.update()

    def add(self):
        if self.hearts_left < 5:
            self.hearts_left += 1




import pygame
import math
import random

from locals import *

import util

def init():
    Cloud.images = []
    Cloud.images.append(util.load_image("cloud1"))
    Cloud.images.append(util.load_image("cloud2"))
    Cloud.images.append(util.load_image("cloud3"))
    Cloud.images.append(util.load_image("cloud4"))

def update():
    if Cloud.t % 150 == 0:
        Cloud.global_clouds.append(Cloud())
        Cloud.cloud_sprites.add(Cloud.global_clouds[-1])
    for cloud in Cloud.global_clouds:
        cloud.update()
        if cloud.rect.right < 0:
            Cloud.global_clouds.remove(cloud)
            Cloud.cloud_sprites.remove(cloud)

    Cloud.t += 1

def draw(screen):
    Cloud.cloud_sprites.draw(screen)

class Cloud (pygame.sprite.Sprite):
    images = None
    global_clouds = []
    cloud_sprites = pygame.sprite.Group()
    t = 0
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)

        if not Cloud.images:
            cloud.init()

        screen = pygame.display.get_surface()
        self.area = screen.get_rect()
        self.cloudtype = int(random.random() * len(Cloud.images))

        self.image = Cloud.images[self.cloudtype]

        self.rect = pygame.Rect(SCREEN_WIDTH, random.random()*70, self.image.get_width(), self.image.get_height())
        self.life = 0

        self.dx, self.dy = -1, 0

    def update(self):
        self.rect.top = self.rect.top + self.dy
        self.rect.left = self.rect.left + self.dx



import pygame

from locals import *

import util

class Score(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)

        self.score = 0
        self.target_score = 0

        self.image = util.smallfont.render("Score: 0", Variables.alpha, (10,10,10))

        self.rect = self.image.get_rect()
        self.rect.left = 100
        self.rect.top = 5

    def update(self):
        if self.target_score > self.score:
            self.score += 1
            self.image = util.smallfont.render("Score: " + str(self.score), Variables.alpha, (10,10,10))
            self.rect.width = self.image.get_width()
            self.rect.height = self.image.get_height()

    def add(self, points):    
        self.target_score += points
        self.update()

    def get_score(self):
        return self.target_score



import pygame
import math

from water import Water

import util

from locals import *

def init():
    Steamboat.image = util.load_image("laiva")
    Steamboat.death_sound = util.load_sound("blub")

class Steamboat(pygame.sprite.Sprite):
    image = None
    death_sound = None
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)

        if not Steamboat.image or not Steamboat.death_sound:
            init()

        self.image = Steamboat.image
        self.rect = pygame.Rect(50, 20, self.image.get_width(), self.image.get_height())
        self.area = pygame.Rect(100, 100, 100, 100)

        #self.hitmask = pygame.surfarray.array_alpha(self.image)

        self.health = 5
        self.jumping = False
        self.taking_damage = False
        self.moving_right = False
        self.moving_left = False
        self.shooting = False
        self.angle = 0
        self.target_angle = 0
        self.dying = False
        self.dead = False
        self.splash = False
        self.blinks=0

        self.steam = []

        self.dx, self.dy = 0,0
        self.t = 0

    def update(self):
        water_levels = [Water.global_water.get_water_level(self.rect.left),
                        Water.global_water.get_water_level(self.rect.centerx),
                        Water.global_water.get_water_level(self.rect.right)]
        self.splash = False

        if self.dying:
            angle = 0.1 * self.target_angle + 0.9 * self.angle

            self.update_angle(angle)
            self.rect.top += self.dy
            self.dy += 1

            if self.rect.bottom > water_levels[1]:
                self.dy *= 0.8

            if self.rect.top >= SCREEN_HEIGHT:
                self.dead = True
            return

        if self.moving_left and not self.moving_right:
            self.dx = -2
        elif self.moving_right:
            self.dx = 2
        else:
            self.dx = 0

        if self.rect.bottom > water_levels[1]:
            if self.jumping:
                self.splash = True
            self.jumping = False
            plap = self.rect.bottom - water_levels[1]
            self.dy *= 0.8
            if self.rect.top > water_levels[1]:
                self.dy -= 2
            else:
                self.dy -= (self.rect.bottom - water_levels[1]) * 0.25

            self.target_angle = 180.0 / math.pi * math.atan((water_levels[0] - water_levels[2]) / 32.0) + math.sin(self.t * 0.05) * 5
        else:
            self.jumping = True

        self.dy += 1

        self.rect.left += self.dx
        self.rect.top += self.dy #move((self.dx, self.dy))

        if self.rect.left < 0:
            self.rect.left = 0
        if self.rect.right > SCREEN_WIDTH:
            self.rect.right = SCREEN_WIDTH

        self.t += 1
        self.update_angle(self.angle * 0.8 + self.target_angle * 0.2)
        
        if self.blinks:
            if self.blinks&1:
                self.image.fill((255,255,255,0))
            self.blinks-=1

    def update_angle(self, angle):
        self.angle = angle
        self.image = util.rotate(Steamboat.image, angle)
        self.rect.width = self.image.get_width()
        self.rect.height = self.image.get_height()
        #self.hitmask = pygame.surfarray.array_alpha(self.image)

    def jump(self):
        if not self.dying:
            if not self.jumping:
                self.jumping = True
                self.dy = -10
                #self.target_angle += 45
                #self.update_angle(-25)

    def move_left(self, bool):
        if not self.dying:
            if bool:
                self.moving_left = True
            else:
                self.moving_left = False

    def move_right(self, bool):
        if not self.dying:
            if bool:
                self.moving_right = True
            else:
                self.moving_right = False

    def get_point(self, point):
        dx = point[0] - self.image.get_rect().centerx
        dy = point[1] - self.image.get_rect().centery

        new_point = [-dy * math.sin(-math.pi / 180.0 * self.angle) + dx * math.cos(-math.pi / 180.0 * self.angle),
                      dy * math.cos(-math.pi / 180.0 * self.angle) + dx * math.sin(-math.pi / 180.0 * self.angle)]
        
        return new_point

    def die(self):
        if not self.dying:
            if (Variables.sound):
               Steamboat.death_sound.play(3, 0)
            self.dying = True
            self.dy = -5
            self.target_angle = -90



from pygame.locals import *

# Some variables used throughout the code

NEXTFRAME = USEREVENT + 1
MIN_FIRE_DELAY = 10
SCREEN_WIDTH = 400#320
SCREEN_HEIGHT = 300#240
SCREEN_FULLSCREEN = 0
FPS = 30


class Variables:
    aa = True
    alpha = True
    particles = True
    sound = True    
    music = True
    name = "Funny Boater"



import pygame
from pygame.locals import *

def _pixelPerfectCollisionDetection(sp1,sp2):
    """
    Internal method used for pixel perfect collision detection.
    """
    rect1 = sp1.rect;     
    rect2 = sp2.rect;                            
    rect  = rect1.clip(rect2)
                    
    #hm1 = sp1.hitmask
    #hm2 = sp2.hitmask
            
    x1 = rect.x-rect1.x
    y1 = rect.y-rect1.y
    x2 = rect.x-rect2.x
    y2 = rect.y-rect2.y

    for r in xrange(0,rect.height):      
        for c in xrange(0,rect.width):
            # I changed this for the 1.5 release of Funny Boat,
            # because generating hitmasks is a bit problematic
            # now that Numeric is outdated and surfarrays don't work.
            # It's not an optimal solution, but is good enough for
            # this game.

            #if hm1[c+x1][r+y1] & hm2[c+x2][r+y2]:
            if sp1.image.get_at((c+x1, r+y1))[3] & sp2.image.get_at((c+x2, r+y2))[3]:
                return 1        

    return 0

def spritecollide_pp(sprite, group, dokill):
    """pygame.sprite.spritecollide_pp(sprite, group, dokill) -&gt; list
       pixel perfect collision detection between sprite and group

       given a sprite and a group of sprites, this will
       return a list of all the sprites that intersect
       the given sprite.
       all sprites must have a "hitmap" value, which is a 2d array 
       that contains a value larger than zero for all pixels that 
       can collide. the "hitmap" 2d array can be set by using 
       pygame.surfarray.array_colorkey() or pygame.surfarray.array_alpha(). 
       all sprites must have a "rect" value, which is a
       rectangle of the sprite area.if the dokill argument
       is true, the sprites that do collide will be
       automatically removed from all groups."""
    crashed = []
    spritecollide = sprite.rect.colliderect
    ppcollide = _pixelPerfectCollisionDetection
    if dokill:
        for s in group.sprites():
            if spritecollide(s.rect):
                if ppcollide(sprite,s):
                    s.kill()
                    crashed.append(s)
    else:
        for s in group.sprites():
            if spritecollide(s.rect):
                if ppcollide(sprite,s):
                    crashed.append(s)
    return crashed


def groupcollide_pp(groupa, groupb, dokilla, dokillb):
    """pygame.sprite.groupcollide_pp(groupa, groupb, dokilla, dokillb) -&gt; dict
       collision detection between group and group by using pixel perfect 
       collision detection

       given two groups, this will find the intersections
       between all sprites in each group. it returns a
       dictionary of all sprites in the first group that
       collide. the value for each item in the dictionary
       is a list of the sprites in the second group it
       collides with. the two dokill arguments control if
       the sprites from either group will be automatically
       removed from all groups."""
    crashed = {}
    SC = spritecollide_pp
    if dokilla:
        for s in groupa.sprites():
            c = SC(s, groupb, dokillb)
            if c:
                crashed[s] = c
                s.kill()
    else:
        for s in groupa.sprites():
            c = SC(s, groupb, dokillb)
            if c:
                crashed[s] = c
    return crashed

def spritecollideany_pp(sprite, group):
    """pygame.sprite.spritecollideany_pp(sprite, group) -&gt; sprite
       finds any sprites that collide by using pixel perfect 
       collision detection

       given a sprite and a group of sprites, this will
       return return any single sprite that collides with
       with the given sprite. If there are no collisions
       this returns None.

       if you don't need all the features of the
       spritecollide function, this function will be a
       bit quicker.

       all sprites must have a "hitmap" value, which is a 2d array 
       that contains a value larger than zero for all pixels that 
       can collide. the "hitmap" 2d array can be set by using 
       pygame.surfarray.array_colorkey() or pygame.surfarray.array_alpha(). 

       all sprites must have a "rect" value, which is a
       rectangle of the sprite area.       
       """
    spritecollide = sprite.rect.colliderect
    ppcollide = _pixelPerfectCollisionDetection    
    for s in group.sprites():
        if spritecollide(s.rect):
            if ppcollide(sprite,s):
                return s
    return None



import pygame
import random

from water import Water
from locals import *

import util

def init():
    Mine.image = util.load_image("miina")
    Mine.sound = util.load_sound("poks")

class Mine(pygame.sprite.Sprite):
    image = None
    sound = None
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)

        # This is an easy way to get a random number that is a water level.
        height = int(SCREEN_HEIGHT -  Water.global_water.get_water_level(random.random() * 320)) - 4

        if not Mine.image or not Mine.sound:
            mine.init()

        self.image = pygame.Surface((Mine.image.get_width(), Mine.image.get_height() + height), SRCALPHA, 32)
        self.image.fill((0,0,0,0))

        self.rect = self.image.get_rect()
        self.rect.bottom = SCREEN_HEIGHT
        self.rect.left = SCREEN_WIDTH

        self.image.blit(Mine.image, Mine.image.get_rect())
        pygame.draw.line(self.image, (25, 25, 25), (self.image.get_rect().centerx, Mine.image.get_height()),
                                                   (self.image.get_rect().centerx, self.image.get_height()))

        #self.hitmask = pygame.surfarray.array_alpha(self.image)

        self.dx = -1

        self.exploding = False
        self.explode_frames = 0

    def update(self):
        water_level = Water.global_water.get_water_level(self.rect.centerx)

        self.rect.left += self.dx

        if SCREEN_HEIGHT - water_level < self.rect.height - 4:
            self.rect.top = water_level - 4
        else:
            self.rect.bottom = SCREEN_HEIGHT

        if self.exploding:
            if self.explode_frames > 0:
                self.explode_frames -= 1
            # animate

    def explode(self):
        if (Variables.sound):
           Mine.sound.play()

        self.exploding = True
        self.explode_frames = 10




import pygame
import math

from water import Water

import util

from locals import *

def init():
    Titanic.image = util.load_image("titanic")
    Titanic.death_sound = util.load_sound("blub")

class Titanic(pygame.sprite.Sprite):
    image = None
    death_sound = None
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)

        if not Titanic.image or not Titanic.death_sound:
            init()

        self.image = Titanic.image
        self.rect = pygame.Rect(SCREEN_WIDTH, Water.global_water.get_water_level(SCREEN_WIDTH), self.image.get_width(), self.image.get_height())
        self.rect.bottom = self.rect.top
        self.area = pygame.display.get_surface().get_rect()

        #self.hitmask = pygame.surfarray.array_alpha(self.image)

        self.taking_damage = False
        self.shooting = False
        self.angle = 0
        self.targetangle = 0
        self.dying = False
        self.dead = False

        self.health = 100

        self.x, self.y = self.rect.left, self.rect.top
        self.dx, self.dy = -1,0.0
        self.t = 0

    def damage(self, dec=1):
        self.health -= dec

        if self.health == 0:
            self.die()

    def update(self):
        water_levels = [Water.global_water.get_water_level(self.rect.left),
                        Water.global_water.get_water_level(self.rect.centerx),
                        Water.global_water.get_water_level(self.rect.right)]
        self.t += 1

        if self.dying:
            angle = 0.007 * self.target_angle + 0.993 * self.angle

            self.update_angle(angle)
            self.rect.top += self.dy
            self.dy += 1

            if self.rect.bottom > water_levels[1]:
                self.dy *= 0.8

            if self.rect.top >= SCREEN_HEIGHT:
                self.dead = True
            return

        if self.rect.bottom > water_levels[1] + 4:
            self.dy *= 0.8
            if self.rect.top > water_levels[1]:
                self.dy -= 1
            else:
                self.dy -= 0.15 * (self.rect.bottom - water_levels[1])

            self.targetangle = 0.01 * (180.0 / math.pi * math.atan((water_levels[0] - water_levels[2]) / 32.0) + math.sin(self.t * 0.05) * 5)

        self.dy += 1

        if self.rect.right < SCREEN_WIDTH:
            self.dx = 0

        self.rect.left += self.dx
        self.rect.top += self.dy

        self.update_angle(self.angle * 0.8 + self.targetangle * 0.2)

    def update_angle(self, angle):
        self.angle = angle
        self.image = util.rotate(Titanic.image, angle)
        self.rect.width = self.image.get_width()
        self.rect.height = self.image.get_height()
        #self.hitmask = pygame.surfarray.array_alpha(self.image)

    def get_point(self, point):
        dx = point[0] - self.image.get_rect().centerx
        dy = point[1] - self.image.get_rect().centery

        new_point = [-dy * math.sin(-math.pi / 180.0 * self.angle) + dx * math.cos(-math.pi / 180.0 * self.angle),
                      dy * math.cos(-math.pi / 180.0 * self.angle) + dx * math.sin(-math.pi / 180.0 * self.angle)]
        
        return new_point

    def die(self):
        if (Variables.sound):
           Titanic.death_sound.play(3, 0)
        self.dying = True
        self.dy = -5
        self.target_angle = 90



import pygame
import math

from water import Water
from locals import *

import util

class Cannonball (pygame.sprite.Sprite):
    image = None
    spec_image = None
    sound = None
    def __init__(self, ship_rect, ship_angle, left = False, special = False):
        pygame.sprite.Sprite.__init__(self)

        if not Cannonball.image:
            Cannonball.image = util.load_image("kuti")
        if not Cannonball.spec_image:
            Cannonball.spec_image = pygame.transform.flip(util.load_image("lokki2"),1,0)
            
        if not Cannonball.sound:
            Cannonball.sound = util.load_sound("pam")
        screen = pygame.display.get_surface()
        self.area = screen.get_rect()

        if special:
            self.image= Cannonball.spec_image
        else:
            self.image = Cannonball.image
        self.special=special
        self.underwater=0
        
        #self.hitmask = pygame.surfarray.array_alpha(self.image)
        if (Variables.sound):
            Cannonball.sound.play()
        #self.dy = -5
        #self.dx = 10
        # Shoot at an angle of 25 relative to the ship

        self.angle=0
        
        if not left:
            if special:
                velocity = 14.0
                self.rect = pygame.Rect(ship_rect.right, ship_rect.top, self.image.get_width(), self.image.get_height())
                self.vect = [math.cos((-ship_angle - 15.0) / 180.0 * math.pi) * velocity,
                         math.sin((-ship_angle - 15.0) / 180.0 * math.pi) * velocity]
            else:
                velocity = 11.0
                self.rect = pygame.Rect(ship_rect.right, ship_rect.centery, self.image.get_width(), self.image.get_height())
            self.vect = [math.cos((-ship_angle - 25.0) / 180.0 * math.pi) * velocity,
                         math.sin((-ship_angle - 25.0) / 180.0 * math.pi) * velocity]
        else:
            self.rect = pygame.Rect(ship_rect.left, ship_rect.centery, self.image.get_width(), self.image.get_height())
            self.vect = [math.cos((-ship_angle + 180.0 + 25.0) / 180.0 * math.pi) * 11.0,
                         math.sin((-ship_angle + 180.0 + 25.0) / 180.0 * math.pi) * 11.0]
        # Will have to think this through later
        #self.vect = [10, -2] #vect

    def update_angle(self, angle):
        self.angle = angle
        self.image = util.rotate(Cannonball.spec_image, angle)
        self.rect.width = self.image.get_width()
        self.rect.height = self.image.get_height()
        #self.hitmask = pygame.surfarray.array_alpha(self.image)

    def tail_point(self):
        return self.rect.left, self.rect.centery-3+self.rect.width*math.sin(self.angle/180.0*math.pi)

    def update(self):
        self.rect.left += self.vect[0] #self.dx
        self.rect.top += self.vect[1] #self.dy



        self.vect[1] += 0.4
        #self.dy += 1
        if self.rect.bottom > Water.global_water.get_water_level(self.rect.centerx):
            
            self.vect[0] *= 0.9
            self.vect[1] *= 0.9
            self.underwater=1

        if 1 and self.special:
            try: 
                self.update_angle(-180/math.pi*math.atan(self.vect[1]/self.vect[0]))
            except ZeroDivisionError:
                self.update_angle(-90)




import pygame

from locals import *

import util
import random

def init():
    Seagull.animation = []
    Seagull.animation.append(util.load_image("lokki1"))
    Seagull.animation.append(util.load_image("lokki2"))
    Seagull.animation.append(util.load_image("lokki3"))
    Seagull.death_sound = util.load_sound("kraah")

class Seagull (pygame.sprite.Sprite):
    animation = None
    death_sound = None
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)

        if not Seagull.animation or not Seagull.death_sound:
            seagull.init()

        self.image = Seagull.animation[0]
        #self.hitmask = pygame.surfarray.array_alpha(self.image)

        self.rect = self.image.get_rect()

        self.rect.top = SCREEN_HEIGHT / 10.0 + random.random() * SCREEN_HEIGHT / 10.0
        self.rect.left = SCREEN_WIDTH
        self.area = pygame.display.get_surface().get_rect()

        self.vect = [-2.0, 0.0]

        self.angle = 0
        self.target_angle = 0

        self.t = 0

        self.dying = False
        self.dead = False

        self.frame = 0

    def update(self):
        self.t += 1

        if not self.dying:
            if self.t % 3 == 0:
                self.frame += 1
                if self.frame >= len(Seagull.animation):
                    self.frame = 0

        self.rect.left += self.vect[0]
        self.rect.top += self.vect[1]

        #if (self.t / 3) % len(Seagull.animation) == 2:
        #self.vect[1] -= 5.0

        #if self.angle != self.target_angle:
        self.update_angle(self.angle * 0.8 + self.target_angle * 0.2)

        if self.dying:
            self.vect[1] += 1.0
            if not self.rect.colliderect(self.area):
                self.dead = True

    def update_angle(self, angle):
        self.angle = angle
        # seagulls look best when they are sharp
        #~ self.image = util.rotate(Seagull.animation[self.frame], self.angle)
        self.image = pygame.transform.rotate(Seagull.animation[self.frame], self.angle)
        self.rect.width = self.image.get_width()
        self.rect.height = self.image.get_height()
        #self.hitmask = pygame.surfarray.array_alpha(self.image)

    def die(self):
        if (Variables.sound):
           Seagull.death_sound.play()
        self.target_angle = 90
        self.dying = True
        self.vect = [0.0, -3.0]



import pygame
import os
import sys
import codecs

import util
from locals import *
from water import Water

import cloud

class Highscores:
    sky = None
    def __init__(self, screen, new_score = -1, endless = False, online = False):
        self.screen = screen

        if not endless:
            self.title = util.bigfont.render("Story Mode", True, (0,0,0))
        elif not online:
            self.title = util.bigfont.render("Endless Mode", True, (0,0,0))
        else:
            self.title = util.bigfont.render("Endless Online", True, (0,0,0))

        if not Highscores.sky:
            Highscores.sky = util.load_image("taivas")
            Highscores.sky = pygame.transform.scale(Highscores.sky, (SCREEN_WIDTH, SCREEN_HEIGHT))

        self.inputting = False
        self.input_score = -1 

        self.scores = []
        self.done = False

        self.endless = endless

        if online:
            if not self.online_init():
               self.done = True
               print "Could not get online highscores!"
            return

        self.pathname = util.get_config_path()
        if not endless:
            self.filename = self.pathname + "/scores"
        else:
            self.filename = self.pathname + "/endless_scores"

        try:
            if not os.path.exists(self.pathname):
                os.mkdir(self.pathname)
        except:
            print "Can't make directory " + self.pathname
            self.done = True
            return

        if not os.path.exists(self.filename):
            #print "Creating dummy high scores"
            self.dummy_scores()
        else:
            try:
                f = codecs.open(self.filename, "r", "utf_8")
                i = 0
                name, score = "", 0
                for line in f:
                    if i % 2 == 0:
                        name = line.strip()
                    else:
                        try:
                            score = int(line)
                        except:
                            print "Corrupt high score file."
                            self.dummy_scores()
                            break
                        self.scores.append((name, score))
                    i += 1
            except:
                self.dummy_scores()
                print "Can't open file " + self.filename + " or file corrupt"

        if len(self.scores) < 10:
            print "Corrupt high score file."
            self.dummy_scores()

        # The online highscore is always submitted
        if endless and new_score != -1:
            self.submit_score(Variables.name, new_score)

        if new_score > self.scores[9][1]:
            #print "It's a new high score!"
            #self.inputting = True
            for i in xrange(10):
                if self.scores[i][1] < new_score:
                    self.input_score = i
                    for j in xrange(9 - i):
                        self.scores[9 - j] = self.scores[8 - j]
                    self.scores[i] = [Variables.name, new_score]
                    break

            self.write_scores()

    def online_init(self):
        HIGHSCORE_URL = "http://funnyboat.sourceforge.net/cgi-bin/top10.py"
        try:
            import socket
            socket.setdefaulttimeout(20)
            import urllib2
            f = urllib2.urlopen(HIGHSCORE_URL)
            for line in f.readlines():
                name,score = line.split("\t")
                score = int(score)
                name = unicode(name, "utf-8")
                self.scores.append((name,score))
            f.close()
        except:
            # Getting highscores failed
            return False
        # Success
        return True

    def submit_score(self, name, score):
            SUBMIT_URL = "http://funnyboat.sourceforge.net/cgi-bin/submit.py"
            try:
                import socket
                socket.setdefaulttimeout(20)
                import urllib
                import urllib2
                #name,score = self.scores[self.input_score]
                data = urllib.urlencode({'name' : name.encode('utf-8'), 'score' : score})
                req = urllib2.Request(SUBMIT_URL, data)
                f = urllib2.urlopen(req)
                f.close()
            except:
                print "Failed to submit highscore"

    def run(self):
        water = Water.global_water
        water.set_amplitude(SCREEN_HEIGHT / 8.0)
        water_sprite = pygame.sprite.Group()
        water_sprite.add(water)
        while not self.done:
            self.screen.blit(Highscores.sky, self.screen.get_rect())
            water.update()
            cloud.update()
            cloud.draw(self.screen)
            water_sprite.draw(self.screen)

            rect = self.title.get_rect()
            rect.centerx = self.screen.get_rect().centerx
            rect.top = 10

            self.screen.blit(self.title, rect)

            for i in xrange(10):
                color = (0,0,0)
                #if self.inputting and self.input_score == i:
                if self.input_score == i:
                    color = (220, 120, 20)
                score = self.scores[i]
                image = 0
                try:
                    image = util.smallfont.render(str(i + 1) + ". " + score[0], True, color)
                except:
                    image = util.smallfont.render(str(i + 1) + ". Error", True, color)
                rect = image.get_rect()
                rect.top = 50 + i * 1.5 * rect.height
                rect.left = 10
                self.screen.blit(image, rect)

                image = util.smallfont.render(str(score[1]), True, color)
                rect = image.get_rect()
                rect.top = 50 + i * 1.5 * rect.height
                rect.right = self.screen.get_rect().right - 10
                self.screen.blit(image, rect)

            pygame.display.flip()

            nextframe = False
            while not nextframe:
                pygame.event.post(pygame.event.wait())
                for event in pygame.event.get():
                    if event.type == NEXTFRAME:
                        nextframe = True
                        continue
                    if self.inputting:
                        if event.type == QUIT:
                            self.inputting = False
                            self.write_scores()
                        if event.type == KEYDOWN:
                            if event.key == K_RETURN or event.key == K_ESCAPE:
                                self.inputting = False
                                if self.endless:
                                    self.submit_score()
                                self.write_scores()
                            elif event.key == K_BACKSPACE:
                                if len(self.scores[self.input_score][0]) != 0:
                                    self.scores[self.input_score][0] = self.scores[self.input_score][0][:-1]
                            elif event.key == K_SPACE or event.unicode != " ":
                                if len(self.scores[self.input_score][0]) < 32:
                                    self.scores[self.input_score][0] += event.unicode
                    else:
                        if event.type == KEYDOWN or event.type == QUIT or event.type == JOYBUTTONDOWN:
                            self.done = True
                            nextframe = True

    def dummy_scores(self):
        self.scores = []
        self.scores.append(("Funny Boat",     2000)) # 1
        self.scores.append(("Hectigo",        1500)) # 2
        self.scores.append(("JDruid",         1000)) # 3
        self.scores.append(("Pekuja",          750)) # 4
        self.scores.append(("Pirate",          500)) # 5
        self.scores.append(("Shark",           400)) # 6
        self.scores.append(("Seagull",         300)) # 7
        self.scores.append(("Naval mine",      200)) # 8
        self.scores.append(("Cannonball",      100)) # 9
        self.scores.append(("Puffy the Cloud",  50)) #10

        self.write_scores()

    def write_scores(self):
        try:
            f = codecs.open(self.filename, "w", "utf_8")
            for i in xrange(10):
                print >> f, self.scores[i][0]
                print >> f, self.scores[i][1]
        except:
            print "Failed to write high scores to file " + self.filename
            self.done = True
            return



import pygame
import pygame
import math
import random
import sys
import os

import PixelPerfect

from pygame.locals import *

from shark import Shark
from health import Health
from cannonball import Cannonball
from steamboat import Steamboat
from particles import Particles
from mine import Mine
from score import Score
from water import Water
from pirateboat import Pirateboat
from seagull import Seagull
from level import Level
from titanic import Titanic
from powerup import Powerup

import cloud

import util

from locals import *

rr=random.random

def rrr(left,right):
    return left+rr()*(right-left)
    
class Game:
    sky = None
    
    
    
        

    def __init__(self, screen, endless = False):
        self.screen = screen

        self.sharks = []
        self.shark_sprites = pygame.sprite.Group()

        self.player = Steamboat()
        self.player_sprite = pygame.sprite.Group()
        self.player_sprite.add(self.player)

        self.health = Health()
        self.health_sprite = pygame.sprite.Group()
        self.health_sprite.add(self.health)

        self.damage_count = 0

        self.t = 0

        self.water = Water.global_water
        #Water.global_water = self.water
        self.water_sprite = pygame.sprite.Group()
        self.water_sprite.add(self.water)

        if not Game.sky:
            Game.sky = util.load_image("taivas")
            Game.sky = pygame.transform.scale(self.sky, (SCREEN_WIDTH, SCREEN_HEIGHT))

        self.cannonballs = []
        self.cannonball_sprites = pygame.sprite.Group()

        self.pirates = []
        self.pirate_sprites = pygame.sprite.Group()

        self.titanic = None
        self.titanic_sprite = pygame.sprite.Group()

        self.seagulls = []
        self.seagull_sprites = pygame.sprite.Group()

        self.particles = Particles()
        self.particle_sprite = pygame.sprite.Group()
        self.particle_sprite.add(self.particles)

        self.mines = []
        self.mine_sprites = pygame.sprite.Group()

        self.score = Score()
        self.score_sprite = pygame.sprite.Group()
        self.score_sprite.add(self.score)

        self.powerups = []
        self.powerup_sprites = pygame.sprite.Group()

        self.level = Level(endless)

        self.lastshot = MIN_FIRE_DELAY + 1

        self.gameover = False
        self.gameover_image = None
        self.gameover_rect = None
        self.done = False

        self.pause = False
        self.pause_image = util.bigfont.render("Pause", Variables.alpha, (0,0,0))
        self.pause_rect = self.pause_image.get_rect()
        self.pause_rect.center = self.screen.get_rect().center

        self.spacepressed = None

    def run(self):
        while not self.done:
            if not self.pause:
                if not self.gameover:
                    self.spawn_enemies()

                self.update_enemies()
                self.player.update()
                self.health.update()
                cloud.update()
                for cb in self.cannonballs:
                    if not cb.underwater:
                        #~ particle_point=cb.rect.left, cb.rect.top
                        particle_point = cb.tail_point()
                        self.particles.add_trace_particle(particle_point)
                        particle_point = [particle_point[i] + cb.vect[i]/2 for i in (0,1)]
                        self.particles.add_trace_particle(particle_point)
                    if cb.special and (not cb.underwater or rr()>0.6) :
                        particle_point = cb.tail_point()
                        self.particles.add_explosion_particle(particle_point)
                    und_old=cb.underwater
                    cb.update()
                    if cb.underwater and not und_old:
                        for i in xrange(5):
                            particle_point=cb.rect.right-4.0+rr()*8.0, cb.rect.top+rr()*2.0
                            self.particles.add_water_particle(particle_point)
                    if (cb.rect.right < 0 and cb.vect[0] < 0) or (cb.rect.left > SCREEN_WIDTH and cb.vect[0] > 0) or (cb.rect.top > SCREEN_HEIGHT):
                        self.cannonballs.remove(cb)
                        self.cannonball_sprites.remove(cb)
                self.score.update()

                # Add steam particles
                if Variables.particles:
                    particle_point = self.player.get_point((5.0 + rr() * 9.0, 0))
                    particle_point[0] += self.player.rect.centerx
                    particle_point[1] += self.player.rect.centery
                    if self.spacepressed and self.t > self.spacepressed + FPS * 3:
                        pass
                        #~ self.particles.add_fire_steam_particle(particle_point)
                    else:
                        self.particles.add_steam_particle(particle_point)

                    particle_point = self.player.get_point((19.0 + rr() * 7.0, 5.0))
                    particle_point[0] += self.player.rect.centerx
                    particle_point[1] += self.player.rect.centery
                    if self.spacepressed and self.t > self.spacepressed + FPS * 3:
                        pass
                        #~ self.particles.add_fire_steam_particle(particle_point)
                    else:
                        self.particles.add_steam_particle(particle_point)

                    if self.titanic:
                        for j in xrange(4):
                            particle_point = self.titanic.get_point((49 + rr() * 9.0 + 28 * j, 25))
                            particle_point[0] += self.titanic.rect.centerx
                            particle_point[1] += self.titanic.rect.centery
                            self.particles.add_steam_particle(particle_point)

                    self.particles.update()

                self.water.update()

                if self.player.splash:
                    if Variables.particles:
                        for i in xrange(10):
                            r = rr()
                            x = int(r * self.player.rect.left + (1.0-r) * self.player.rect.right)
                            point = (x, self.water.get_water_level(x))
                            self.particles.add_water_particle(point)
    
                for powerup in self.powerups:
                    powerup.update()
                    if powerup.picked:
                        self.powerups.remove(powerup)
                        self.powerup_sprites.remove(powerup)

                if not self.gameover:
                    self.check_collisions()

                if self.health.hearts_left == 0 and not self.player.dying:
                    self.player.die()

                if self.player.dying and not self.player.dead:
                    if Variables.particles:
                        self.particles.add_explosion_particle((self.player.rect.centerx, self.player.rect.centery))
                        self.particles.add_debris_particle((self.player.rect.centerx, self.player.rect.centery))

                if self.player.dead:
                    #self.done = True
                    self.set_gameover()

                if self.damage_count > 0:
                    self.damage_count -= 1
                self.lastshot += 1
                self.t += 1

            self.draw()

            self.handle_events()

        return self.score.get_score()

    def damage_player(self):
        self.health.damage()
        for i in xrange(10):
            particle_point = self.player.get_point((rr() * 26.0, rr() * 10.0))
            particle_point[0] += self.player.rect.centerx
            particle_point[1] += self.player.rect.centery
            self.particles.add_debris_particle(particle_point)
        self.player.blinks+=12


    def set_gameover(self, message = "Game Over"):
        self.gameover = True
        images = []
        height = 0
        width = 0
        for text in message.split("\n"):
            images.append(util.bigfont.render(text, Variables.alpha, (0,0,0)))
            height += images[-1].get_height()
            if images[-1].get_width() > width:
                width = images[-1].get_width()
        self.gameover_image = pygame.Surface((width, height), SRCALPHA, 32)
        self.gameover_image.fill((0,0,0,0))
        for i in xrange(len(images)):
            rect = images[i].get_rect()
            rect.top = i * images[i].get_height()
            rect.centerx = width / 2
            self.gameover_image.blit(images[i], rect)

        self.gameover_rect = self.gameover_image.get_rect()
        self.gameover_rect.center = self.screen.get_rect().center

    def take_screenshot(self):
        i = 1
        filename = "sshot.tga"
        while os.path.exists(filename):
            i += 1
            filename = "sshot" + str(i) + ".tga"
        
        pygame.image.save(self.screen, filename)
        print "Screenshot saved as " + filename

    def handle_events(self):
        nextframe = False
        framecount = 0
        while not nextframe:
          # wait until there's at least one event in the queue
          #nextframe = True
          pygame.event.post(pygame.event.wait())
          for event in pygame.event.get():
            #event = pygame.event.wait()
            if event.type == QUIT or \
               event.type == KEYDOWN and event.key == K_ESCAPE:
                self.done = True
                nextframe = True
            elif event.type == NEXTFRAME:
                framecount += 1
                nextframe = True
            elif self.gameover:
                if event.type == JOYBUTTONDOWN:
                    self.done = True
                    nextframe = True
                elif event.type == KEYDOWN:
                    self.done = True
                    nextframe = True
                continue
            elif event.type == JOYAXISMOTION:
                if event.axis == 0:
                    if event.value < -0.5:
                        self.player.move_left(True)
                    elif event.value > 0.5:
                        self.player.move_right(True)
                    else:
                        self.player.move_left(False)
                        self.player.move_right(False)
            elif event.type == JOYBUTTONDOWN:
                if event.button == 0:
                    if not self.pause:
                        self.player.jump()
                elif event.button == 1:
                    if not self.pause:
                        if self.lastshot > MIN_FIRE_DELAY and not self.player.dying:
                            cb = Cannonball(self.player.rect, self.player.angle)
                            self.cannonballs.append(cb)
                            self.cannonball_sprites.add(cb)
                            particle_point = self.player.get_point((42.0,10.0))
                            particle_point[0] += self.player.rect.centerx
                            particle_point[1] += self.player.rect.centery
                            for i in xrange(4):
                                self.particles.add_fire_steam_particle(particle_point)
                            self.lastshot = 0
                            self.spacepressed = self.t
                elif event.button == 5:
                    self.take_screenshot()
                elif event.button == 8:
                    self.set_pause()
            elif event.type == KEYDOWN:
                if event.key == K_LEFT:
                    self.player.move_left(True)
                elif event.key == K_RIGHT:
                    self.player.move_right(True)
                elif event.key == K_SPACE:
                    if not self.pause:
                    # Only 3 cannonballs at once
                    # Maximum firing rate set at the top
                        if self.lastshot > MIN_FIRE_DELAY and not self.player.dying:
                            cb = Cannonball(self.player.rect, self.player.angle)
                            self.cannonballs.append(cb)
                            self.cannonball_sprites.add(cb)
                            particle_point = self.player.get_point((42.0,10.0))
                            particle_point[0] += self.player.rect.centerx
                            particle_point[1] += self.player.rect.centery
                            for i in xrange(4):
                                self.particles.add_fire_steam_particle(particle_point)
                            self.lastshot = 0
                            self.spacepressed = self.t
                elif event.key == K_UP:
                    if not self.pause:
                        self.player.jump()
                elif event.key == K_s:
                    self.take_screenshot()
                elif event.key == K_p:
                    self.set_pause()
            elif event.type == KEYUP:
                if event.key == K_LEFT:
                    self.player.move_left(False)
                elif event.key == K_RIGHT:
                    self.player.move_right(False)
                elif event.key == K_SPACE:
                    if not self.pause:
                        if self.spacepressed and self.t > self.spacepressed + FPS * 3 and not self.player.dying:
                            cb = Cannonball(self.player.rect, self.player.angle, special=True)
                            self.cannonballs.append(cb)
                            self.cannonball_sprites.add(cb)
                            particle_point = self.player.get_point((42.0,10.0))
                            particle_point[0] += self.player.rect.centerx
                            particle_point[1] += self.player.rect.centery
                            for i in xrange(30):
                                self.particles.add_fire_steam_particle((particle_point[0]+rrr(-4,4),particle_point[1]+rrr(-3,3)))
                            self.lastshot = 0
                        self.spacepressed = None
            elif event.type == JOYBUTTONUP:
                if event.button == 1:
                    if not self.pause:
                        if self.spacepressed and self.t > self.spacepressed + FPS * 3 and not self.player.dying:
                            cb = Cannonball(self.player.rect, self.player.angle, special=True)
                            self.cannonballs.append(cb)
                            self.cannonball_sprites.add(cb)
                            particle_point = self.player.get_point((42.0,10.0))
                            particle_point[0] += self.player.rect.centerx
                            particle_point[1] += self.player.rect.centery
                            for i in xrange(30):
                                self.particles.add_fire_steam_particle((particle_point[0]+rrr(-4,4),particle_point[1]+rrr(-3,3)))
                            self.lastshot = 0
                        self.spacepressed = None

        #if framecount > 1:
        #    print str(self.t) + ": missed " + str(framecount - 1) + " frames!"

    def set_pause(self):
        self.pause = not self.pause

        #if framecount > 1:
        #    print str(self.t) + ": missed " + str(framecount - 1) + " frames!"

    def set_pause(self):
        self.pause = not self.pause

    def draw(self):
        self.screen.blit(Game.sky, self.screen.get_rect())
        self.health_sprite.draw(self.screen)
        self.score_sprite.draw(self.screen)
        self.player_sprite.draw(self.screen)
        self.powerup_sprites.draw(self.screen)
        self.pirate_sprites.draw(self.screen)
        if self.titanic:
            self.titanic_sprite.draw(self.screen)
        self.seagull_sprites.draw(self.screen)
        cloud.draw(self.screen)
        self.shark_sprites.draw(self.screen)
        self.mine_sprites.draw(self.screen)
        self.cannonball_sprites.draw(self.screen)
        self.water_sprite.draw(self.screen)
        if Variables.particles:
            self.particle_sprite.draw(self.screen)

        if self.pause:
            self.screen.blit(self.pause_image, self.pause_rect)

        if self.gameover:
            self.screen.blit(self.gameover_image, self.gameover_rect)

        if self.level.t < 120:
            image = None
            i = 0
            if self.level.phase < len(self.level.phase_messages):
              for text in self.level.phase_messages[self.level.phase].split("\n"):
                image = util.smallfont.render(text, Variables.alpha, (0,0,0))
                rect = image.get_rect()
                rect.centerx = self.screen.get_rect().centerx
                rect.top = 100 + rect.height * i
                blit_image = pygame.Surface((image.get_width(), image.get_height()))
                blit_image.fill((166,183,250))
                blit_image.set_colorkey((166,183,250))
                blit_image.blit(image, image.get_rect())
                if self.level.t > 60:
                    blit_image.set_alpha(255 - (self.level.t - 60) * 255 / 60)
                self.screen.blit(blit_image, rect)
                i += 1

        pygame.display.flip()



    def check_collisions(self):
        collisions = PixelPerfect.spritecollide_pp(self.player, self.powerup_sprites, 0)
        for powerup in collisions:
            if not powerup.fading:
                if not self.player.dying:
                    self.health.add()
                    powerup.pickup()

        collisions = PixelPerfect.spritecollide_pp(self.player, self.mine_sprites, 0)

        for mine in collisions:
            if not mine.exploding:
                if not self.player.dying:
                    self.damage_player()
                    mine.explode()

        collisions = PixelPerfect.spritecollide_pp(self.player, self.shark_sprites, 0)

        for shark in collisions:
            if not shark.dying:
                if not self.player.dying:
                    self.damage_player()
                    shark.die()

        collisions = PixelPerfect.spritecollide_pp(self.player, self.cannonball_sprites, 0)

        for cb in collisions:
            if not self.player.dying:
                self.damage_player()
                self.cannonballs.remove(cb)
                self.cannonball_sprites.remove(cb)
                if (Variables.sound):
                   Mine.sound.play() # Umm... the mine has a nice explosion sound.

        collisions = PixelPerfect.groupcollide_pp(self.cannonball_sprites, self.shark_sprites, 0, 0)

        for cb in dict.keys(collisions):
            for shark in collisions[cb]:
                # The test on cb.vect is a rude hack preventing cannonballs from pirate ships from killing sharks.
                if not shark.dying and cb.vect[0] > 0:
                    self.score.add(15)
                    shark.die()
                    # give her a part of ball's momentum:
                    shark.dx+=cb.vect[0]*0.6
                    shark.dy+=cb.vect[1]*0.4
                    if not cb.special:
                        self.cannonballs.remove(cb)
                        self.cannonball_sprites.remove(cb)
                    break

        collisions = PixelPerfect.groupcollide_pp(self.cannonball_sprites, self.seagull_sprites, 0, 0)

        for cb in dict.keys(collisions):
            for seagull in collisions[cb]:
                # cb.vect test is a rude hack preventing pirates from killing seagulls
                if not seagull.dying and cb.vect[0] > 0:
                    self.score.add(75)
                    seagull.die()
                    # give her a part of ball's momentum:
                    seagull.vect[0]+=cb.vect[0]*0.4
                    seagull.vect[1]+=cb.vect[1]*0.4
                    if not cb.special:
                        self.cannonballs.remove(cb)
                        self.cannonball_sprites.remove(cb)
                    break

        collisions = PixelPerfect.groupcollide_pp(self.cannonball_sprites, self.pirate_sprites, 0, 0)

        for cb in dict.keys(collisions):
            for pirate in collisions[cb]:
                # cb.vect hack for preventing pirates from killing each other
                if not pirate.dying and cb.vect[0] > 0:
                    if (Variables.sound):
                       Mine.sound.play() # Umm... the mine has a nice sound.
                    self.score.add(25)
                    pirate.damage()
                    for i in range(6):
                        self.particles.add_wood_particle((pirate.rect.centerx+rrr(-0,15), pirate.rect.centery+rrr(-10,20)))
                    if not cb.special:
                        self.cannonballs.remove(cb)
                        self.cannonball_sprites.remove(cb)
                    break

        if self.titanic:
            collisions = PixelPerfect.spritecollide_pp(self.titanic, self.cannonball_sprites, 0)
            for cb in collisions:
                if not self.titanic.dying and cb.vect[0] > 0:
                    if (Variables.sound):
                       Mine.sound.play()
                    if cb.special: 
                        #special round is hard to fire, so lets reward our crafty player
                        self.titanic.damage(12)
                        self.score.add(100)
                    else:
                        self.score.add(7)
                        self.titanic.damage()
                    self.cannonballs.remove(cb)
                    self.cannonball_sprites.remove(cb)
                    break

    def update_enemies(self):
        self.mine_sprites.update()
        for mine in self.mines:
            if mine.exploding:
                if mine.explode_frames == 0:
                    self.mines.remove(mine)
                    self.mine_sprites.remove(mine)
                # this should really be done in the Mine class, but oh well, here's some explosion effects:
                if Variables.particles:
                    self.particles.add_explosion_particle((mine.rect.centerx, mine.rect.top + mine.image.get_rect().centerx))
                    self.particles.add_debris_particle((mine.rect.centerx, mine.rect.top + mine.image.get_rect().centerx))
            if mine.rect.right < self.screen.get_rect().left:
                self.mines.remove(mine)
                self.mine_sprites.remove(mine)

        self.shark_sprites.update()
        for shark in self.sharks:
            if shark.dying:
                if Variables.particles:
                    self.particles.add_blood_particle(shark.rect.center)
            if shark.rect.right < self.screen.get_rect().left or shark.dead:
                self.sharks.remove(shark)
                self.shark_sprites.remove(shark)

        self.pirate_sprites.update()
        for pirate in self.pirates:
            if pirate.t % 50 == 0 and not pirate.dying:
                # Pirate shoots, this should probably be handled by the Pirateboat class
                cb = Cannonball(pirate.rect, pirate.angle, left = True)
                self.cannonballs.append(cb)
                self.cannonball_sprites.add(cb)
                particle_point = pirate.get_point((0.0,10.0))
                particle_point[0] += pirate.rect.centerx
                particle_point[1] += pirate.rect.centery
                for i in xrange(4):
                    self.particles.add_fire_steam_particle(particle_point)
                
            if pirate.rect.right < self.screen.get_rect().left or pirate.dead:
                self.pirates.remove(pirate)
                self.pirate_sprites.remove(pirate)
            elif pirate.dying:
                if Variables.particles:
                    self.particles.add_explosion_particle((pirate.rect.centerx, pirate.rect.centery))
                    self.particles.add_wood_particle((pirate.rect.centerx, pirate.rect.centery))

        if self.titanic:
            self.titanic.update()
            if self.titanic.t % 100 == 0 and not self.titanic.dying:
                for i in xrange(3):
                    cb = Cannonball(self.titanic.rect, self.titanic.angle + (i-1)*10 - 50, left = True)
                    self.cannonballs.append(cb)
                    self.cannonball_sprites.add(cb)
            elif self.titanic.t % 100 == 50 and not self.titanic.dying:
                for i in xrange(3):
                    cb = Cannonball(self.titanic.rect, self.titanic.angle + (i-1)*10 - 52.5, left = True)
                    self.cannonballs.append(cb)
                    self.cannonball_sprites.add(cb)
            if self.titanic.dead:
                self.set_gameover("Congratulations!\nYou sunk Titanic!")
                self.titanic = None

        self.seagull_sprites.update()
        for seagull in self.seagulls:
            if seagull.rect.right < 0 or seagull.dead:
                self.seagulls.remove(seagull)
                self.seagull_sprites.remove(seagull)

    def spawn_enemies(self):
        spawns = self.level.get_spawns()
        if spawns[Level.SHARKS]:
            # Make a new shark
            self.sharks.append(Shark())
            self.shark_sprites.add(self.sharks[-1])

        if spawns[Level.PIRATES]:
            # Make a new pirate ship
            self.pirates.append(Pirateboat())
            self.pirate_sprites.add(self.pirates[-1])

        if spawns[Level.MINES]:
            self.mines.append(Mine())
            self.mine_sprites.add(self.mines[-1])

        if spawns[Level.SEAGULLS]:
            self.seagulls.append(Seagull())
            self.seagull_sprites.add(self.seagulls[-1])

        if spawns[Level.TITANIC]:
            self.titanic = Titanic()
            self.titanic_sprite.add(self.titanic)

        if spawns[Level.POWERUPS]:
            self.powerups.append(Powerup())
            self.powerup_sprites.add(self.powerups[-1])




import pygame
import random

from pygame.locals import *

import util
import random
from water import Water

class Level:
    SHARKS = 0
    PIRATES = 1
    MINES = 2
    SEAGULLS = 3
    TITANIC = 4
    POWERUPS = 5
    def __init__(self, endless = False):
      self.endless = endless
      if not endless:
        self.phase_lengths = [900, 900, 900, 1800, 1300, -1]
        self.phase_messages = [
                               "Watch out for those\nangry sharks, captain!",
                               "Minefield ahead, captain!",
                               "Oh no! It's the infamous fleet\n of pirate Captain Colorbeard!",
                               "",
                               "Uh, oh. Looks like some busy\nwaters ahead, captain!",
                               "Holy cow! It's the\nlegendary Titanic!"]

        colors = ["Brown", "Red", "Yellow", "Magenta", "Pink", "Cyan", "Blue", "Black", "Green", "Violet", "Beige", "White", "Grey", "Blonde", "Orange", "Brunette", "Ginger", "Turquoise"]
        self.phase_weather = [
                                30.0,
                                40.0,
                                20.0,
                                30.0,
                                60.0,
                                5.0]
        self.phase_messages[2] = "Oh no! It's the infamous fleet\nof pirate Captain " + colors[random.randint(0, len(colors) - 1)] + "beard!"
        # This is the big table describing the phases
        # The subtables are different phases, and they have two-celled tables describing
        # how often different enemies appear in the phase. The first number is the offset
        # from the beginning of the phase to when the enemy first appears, and the second
        # one is the delay between new enemies spawning. Both units are in frames, so 30
        # units equals one second in real time
        self.phases = [[[10, 80], # Shark frequency in first phase.
                        [230, 0], # Pirate frequency in first phase.
                        [120, 0], # Mine frequency in first phase.
                        [0, 3000], # Seagull frequency in first phase.
                        [1, 0], # Titanic
                        [450, 1000]
                        ],
                       [[100, 300], # Second phase
                        [257, 0],
                        [70, 137], # Mines this time around
                        [0, 3000],
                        [1, 0],
                        [0, 1000]
                        ],
                       [[257, 500], # Fewer sharks in the third phase
                        [30, 300], # Yarr, tharr be pirates aboot
                        [470, 500], # A few mines too
                        [0, 1500], # Two seagulls
                        [1, 0],
                        [0, 1000]
                        ],
                       [[0, 183], # Middle phase
                        [230, 319],
                        [40, 217],
                        [0, 700],
                        [1, 0],
                        [0, 1000]
                        ],
                       [[0, 233], # Big Kahuna phase
                        [230, 519],
                        [40, 317],
                        [0, 700],
                        [1, 0],
                        [0, 1000]
                        ],
                       [[70, 200], # The boss level, some sharks, a single powerup and Teh Titanic
                        [300, 0],
                        [0, 200],
                        [0, 0],
                        [10, -1], # -1 means that there will only be one Titanic
                        [0, 0]
                        ]
                        ]
      else:
          self.phase_lengths = [450]
          self.phase_messages = ["This is the endless mode.\nGood luck!"]
          self.phase_weather = [30,10,50]
          self.phases = [
                        [
                        [0, 255], # Balanced phase
                        [150, 257],
                        [50, 253],
                        [0, 507],
                        [0, 0], # No Titanic in the endless mode
                        [100, 0]
                        ],
                        [
                        [0, 150], # Lots of sharks
                        [400, 700],
                        [50, 700],
                        [0, 507],
                        [0, 0],
                        [500, 0]
                        ],
                        [
                        [150, 400], # Lots of pirates
                        [0, 150],
                        [350, 500],
                        [0, 507],
                        [0,0],
                        [500, 0]
                        ],
                        [
                        [350, 500], # Lots of mines
                        [150, 400],
                        [20, 150],
                        [0, 507],
                        [0, 0],
                        [100, -1]
                        ]
                       ]
        
      self.phase = 0
      self.t = 0
      self.modifier = 0

    def get_spawns(self):
        phase_length = self.phase_lengths[self.phase % len(self.phase_lengths)]

        if self.endless:
            phase_length -= self.modifier
            if phase_length < 30:
                phase_length = 30

        if phase_length != -1 and self.t > phase_length:
            self.t = 0
            self.phase += 1

        spawns = []

        for enemy in self.phases[self.phase % len(self.phases)]:
            offset, delay = enemy
            if self.endless and delay > 0:
                delay -= self.phase / 4 * 5
                offset -= self.phase / 4 * 5
                if delay <= 30:
                    delay = 30
                if offset < 0:
                    offset = 0
            if (delay == -1 and self.t == offset) or (delay != 0 and self.t % delay == offset):
                spawns.append(True)
            else:
                spawns.append(False)

        if not self.endless:
            Water.global_water.set_amplitude(self.phase_weather[self.phase % len(self.phase_weather)])
        else:
            Water.global_water.set_amplitude(self.phase_weather[(self.phase / 4) % len(self.phase_weather)])


        self.t += 1

        return spawns



import pygame

from locals import *

import util
import health
from water import Water

class Powerup (pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)

        if not health.Health.heart:
            health.init()

        self.image = health.Health.heart
        #self.hitmask = pygame.surfarray.array_alpha(self.image)

        self.rect = self.image.get_rect()
        self.rect.left = SCREEN_WIDTH
        self.rect.bottom = Water.global_water.get_water_level(SCREEN_WIDTH)

        self.dx, self.dy = -1, 0
        self.t = 0
        self.picked = False
        self.fading = False

    def update(self):
        water_level = Water.global_water.get_water_level(self.rect.centerx)
        if self.fading:
            if self.fadecount > 0:
                self.image.set_alpha(self.fadecount * 255 / 15)
                self.fadecount -= 1
            else:
                self.picked = True
        if self.rect.bottom > water_level:
            self.dy *= 0.8
            if self.rect.top > water_level:
                self.dy -= 2
            else:
                self.dy -= 0.25 * (self.rect.bottom - water_level)

        self.dy += 1

        self.rect.left += self.dx
        self.rect.top += self.dy

        self.t += 1

    def pickup(self):
        self.fading = True
        self.fadecount = 15
        image = pygame.Surface((self.image.get_width(), self.image.get_height()))
        image.fill((255,0,255))
        image.set_colorkey((255,0,255))
        image.blit(self.image, self.image.get_rect())
        self.image = image



import pygame
import math

from locals import *

import util

class Water(pygame.sprite.Sprite):
    global_water = None
    raster_image = None
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)

        self.image = pygame.Surface((SCREEN_WIDTH, SCREEN_HEIGHT))
        self.rect = self.image.get_rect()

        if not Water.raster_image:
            Water.raster_image = util.load_image("rasteri")

        self.water_levels = []
        for i in xrange(self.rect.width):
            self.water_levels.append(0.0)
        self.t = 0


        self.target_amplitude = self.amplitude = SCREEN_HEIGHT / 8.0
        self.target_wavelength = self.wavelength = 0.02 * SCREEN_WIDTH / 2.0 / math.pi
        self.target_speed = self.speed = 0.06 / math.pi / 2.0 * FPS
        self.target_baseheight = self.baseheight = SCREEN_HEIGHT / 24.0 * 8.0

        self.xmul = 2.0 * math.pi / self.wavelength / float(SCREEN_WIDTH)
        self.tmul = math.pi * 2.0 / float(FPS) * self.speed

        self.image=self.image.convert_alpha()
        #~ if Variables.alpha:
            #~ self.image.set_alpha(110)

        self.update()
    
    def update(self):
        self.image.fill((200,210,255,0))
        for x in xrange(self.rect.width):
            h = SCREEN_HEIGHT - (math.sin(x * self.xmul + self.t * self.tmul) * self.amplitude + self.baseheight)
            self.water_levels[x] = h
            h_float,h_int=math.modf(h)
            pygame.draw.line(self.image, (20, 60, 180,110), (x, h_int+1), (x, SCREEN_HEIGHT))
            if Variables.aa:
                self.image.set_at((x,int(h_int)),(20, 60, 180,int(110*(1.0-h_float))))

        if self.target_amplitude != self.amplitude:
            self.amplitude = 0.99 * self.amplitude + 0.01 * self.target_amplitude
        if self.target_wavelength != self.wavelength:
            self.wavelength = 0.99 * self.wavelength + 0.01 * self.target_wavelength
            self.xmul = 2.0 * math.pi / self.wavelength / float(SCREEN_WIDTH)
        if self.target_speed != self.speed:
            self.speed = 0.99 * self.speed + 0.01 * self.target_speed
            self.tmul = math.pi * 2.0 / float(FPS) * self.speed
        if self.target_baseheight != self.baseheight:
            self.baseheight = 0.99 * self.baseheight + 0.01 * self.target_baseheight

        self.t += 1

        if not Variables.alpha:
            self.image.blit(Water.raster_image, self.image.get_rect())
            self.image.set_colorkey((255,0,255))
            

    def get_water_level(self, x):
        if x >= len(self.water_levels):
            return self.water_levels[len(self.water_levels) - 1]
        elif x < 0:
            return self.water_levels[0]
        return self.water_levels[int(x)]
        
    def set_amplitude(self, amplitude):
        self.target_amplitude = amplitude

    def set_wavelength(self, wavelength):
        self.target_wavelength = wavelength

    def set_speed(self, speed):
        self.target_speed = speed

    def set_baseheight(self, baseheight):
        self.target_baseheight = baseheight



import pygame

from water import Water
from locals import *
import cloud
import util

class Options:
    logo = None
    sky = None

    PARTICLES = 0
    AA = 1
    SOUND = 2
    MUSIC = 3
    NAME = 4

    def __init__(self, screen):
        self.screen = screen

        if not Options.sky:
            Options.sky = util.load_image("taivas")
            Options.sky = pygame.transform.scale(Options.sky, (SCREEN_WIDTH, SCREEN_HEIGHT))

        self.water = Water.global_water
        self.water_sprite = pygame.sprite.Group()
        self.water_sprite.add(self.water)

        if not Options.logo:
            Options.logo = util.load_image("logo")

        self.menu = ()
        self.refresh() # sets self.menu
        self.selection = 0
        self.t = 0

    def refresh(self):
        self.menu = ("Particle effects: " + (Variables.particles and "on" or "off"),
                     "Antialiasing: " + (Variables.aa and "on" or "off"),
                     "Sound effects: " + (Variables.sound and "on" or "off"),
                     "Music: " + (Variables.music and "on" or "off"),
                     "Player Name: " + Variables.name)

    def run(self):
        done = False

        while not done:
            self.screen.blit(Options.sky, self.screen.get_rect())
            self.water.update()
            self.water_sprite.draw(self.screen)

            for i in xrange(len(self.menu)):
                self.render(i)

            cloud.update()

            cloud.draw(self.screen)

            rect = Options.logo.get_rect()
            rect.centerx = self.screen.get_rect().centerx
            rect.top = 0
            self.screen.blit(Options.logo, rect)

            image = util.smallfont.render("http://funnyboat.sourceforge.net/", True, (0,0,0))
            rect = image.get_rect()
            rect.midbottom = self.screen.get_rect().midbottom
            self.screen.blit(image, rect)

            pygame.display.flip()

            self.t += 1

            nextframe = False
            while not nextframe:
                pygame.event.post(pygame.event.wait())
                for event in pygame.event.get():
                    if event.type == QUIT or \
                        event.type == KEYDOWN and event.key == K_ESCAPE:
                        self.selection = -1
                        done = True
                        nextframe = True
                    elif event.type == NEXTFRAME:
                        nextframe = True
                    elif event.type == JOYAXISMOTION:
                        if event.axis == 1:
                            if event.value < -0.5:
                                self.move_up()
                            if event.value > 0.5:
                                self.move_down()
                        elif event.axis == 0:
                            if event.value < -0.5:
                                self.change_left()
                            if event.value > 0.5:
                                self.change_right()
                    elif event.type == JOYBUTTONDOWN:
                        if event.button == 0:
                            #done = True
                            self.change_right()
                        elif event.button == 1:
                            done = True
                    elif event.type == KEYDOWN:
                        if event.key == K_UP:
                            self.move_up()
                        elif event.key == K_DOWN:
                            self.move_down()
                        elif event.key == K_LEFT:
                            self.change_left()
                        elif event.key == K_RIGHT:
                            self.change_right()
                        elif self.selection == Options.NAME:
                            if event.key == K_BACKSPACE:
                                if len(Variables.name) != 0:
                                    Variables.name = Variables.name[:-1]
                            elif event.key == K_SPACE or event.unicode != " " and event.unicode>=u' ':
                                if len(Variables.name) < 32:
                                    Variables.name += event.unicode
                            self.refresh()
                        elif event.key == K_SPACE or event.key == K_RETURN:
                            #done = True
                            self.change_right()

        util.save_config()
        return self.selection

    def change_left(self):
        if self.selection == Options.PARTICLES:
            Variables.particles = not Variables.particles
        if self.selection == Options.AA:
            Variables.aa = not Variables.aa
        elif self.selection == Options.SOUND:
            Variables.sound = not Variables.sound
        elif self.selection == Options.MUSIC:
            Variables.music = not Variables.music
            try:
                if Variables.music:
                    pygame.mixer.music.play(-1)
                else:
                    pygame.mixer.music.stop()
            except:
                pass
        self.refresh()

    def change_right(self):
        # They're all bools right now, so...
        self.change_left()

    def move_down(self):
        self.selection += 1
        if self.selection >= len(self.menu):
            self.selection = len(self.menu) - 1

    def move_up(self):
        self.selection -= 1
        if self.selection < 0:
            self.selection = 0

    def render(self, id):
        color = (0,0,0)
        if self.selection == id:
            color = (220, 120, 20)

        title = self.menu[id]
        image = util.smallfont.render(title, Variables.alpha, color)
        rect = image.get_rect()
        rect.centerx = self.screen.get_rect().centerx
        rect.top = Options.logo.get_height() + id * rect.height * 1.1

        self.screen.blit(image, rect)



import pygame
import pygame
import math

from water import Water

import util

from locals import *

def init():
    Shark.image = util.load_image("hai")
    Shark.death_sound = util.load_sound("kraah")

class Shark (pygame.sprite.Sprite):
    image = None
    death_sound = None
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)

        screen = pygame.display.get_surface()
        self.area = screen.get_rect()

        if not Shark.image or not Shark.death_sound:
            shark.init()

        self.image = Shark.image

        self.rect = pygame.Rect(SCREEN_WIDTH, 0, Shark.image.get_width(), Shark.image.get_height())
        self.life = 0

        #self.hitmask = pygame.surfarray.array_alpha(self.image)

        self.dx, self.dy = -3, 0

        self.jumping = False
        self.dying = False
        self.dead = False
        self.angle = 0
        self.target_angle = 0

    def update(self):
        water_levels = [Water.global_water.get_water_level(self.rect.left),
                        Water.global_water.get_water_level(self.rect.centerx),
                        Water.global_water.get_water_level(self.rect.right)]
        if self.dying:
            angle = 0.4 * self.target_angle + 0.6 * self.angle

            self.update_angle(angle)
            self.rect.top += self.dy
            self.rect.left += self.dx
            self.dy += 1

            if self.rect.bottom > water_levels[1]:
                self.dy *= 0.8
                self.dx *= 0.8

            if self.rect.top >= SCREEN_HEIGHT:
                self.dead = True
            return
        if not self.jumping:
            self.rect.top = water_levels[1] - 8
            self.target_angle = math.atan((water_levels[0] - water_levels[2]) / 32.0) * 180.0 / math.pi
        else:
            self.dy = self.dy + 1
            if self.rect.top > water_levels[1] - 8:
                self.jumping = False
                self.dy = 0
                self.dx = -2

        if self.life % 40 == 0:
            self.jumping = True
            self.dy = -10
            self.dx = - 3

        self.rect.top = self.rect.top + self.dy
        self.rect.left = self.rect.left + self.dx

        self.life = self.life + 1

        angle = 0.2 * self.target_angle + 0.8 * self.angle

        self.update_angle(angle)


    def update_angle(self, angle):
        self.angle = angle

        self.image = util.rotate(Shark.image, angle)
        self.rect.width = self.image.get_width()
        self.rect.height = self.image.get_height()
        #self.hitmask = pygame.surfarray.array_alpha(self.image)

    def die(self):
        self.dying = True
        while not self.dying:
            pass
        if (Variables.sound):
           Shark.death_sound.play()
        self.dy = -5
        self.target_angle = 90
        #self.image = pygame.transform.rotate(Shark.image, 90)

